/* 
 * Copyright (c) 2003 RIKEN (The Institute of Physical and Chemical Research)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY RIKEN AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL RIKEN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: Builtin_Buffer.cpp,v 1.1.1.1 2004/03/31 08:15:07 orrisroot Exp $ */
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "SL_header.h"

using namespace std;

#include <libsatellite.h>

#define  __IMPORTSYMBOL__
#include "SL_exception.h"
#include "history.h"
#include "module.h"
#include "tty_console.h"
#include "SL_Index.h"
#include "Base_Buffer.h"
#include "Series_Buffer.h"
#include "Snapshot_Buffer.h"
#include "String_Buffer.h"
#include "Scalar_Buffer.h"
#include "SL_Tool.h"
#include "SL_Object.h"
#include "SymbolList.h"
#include "SystemCommon.h"
#undef   __IMPORTSYMBOL__
#include "Program.h"
#include "Builtin.h"
#include "Builtin_Buffer.h"

static int bb_cnt=0;

DLLEXPORT void print_builtin_buffer_num(){
  printf("Buitin_Buffer res:%d\n",bb_cnt);
}

Builtin_Buffer::Builtin_Buffer() { 
  bbtype=BLTINBUF_TYPE_UNDEF; type=BUILTIN_B; bb_cnt++;
}

Builtin_Buffer::~Builtin_Buffer(){ 
  FreeBuffer(); bb_cnt--; 
}

Base_Buffer *Builtin_Buffer::duplicate(){
  Base_Buffer *ret;
  try{
    ret = new Builtin_Buffer();
  }catch(bad_alloc){
    return 0;
  }
  ret->CopyBuffer(this);
  return ret;
}

bool Builtin_Buffer::FreeBuffer(){
  switch(bbtype){
  case BLTINBUF_TYPE_SPROG:
    if(data) delete (SubProgram*)data;
    break;
  case BLTINBUF_TYPE_MODULE:
    if(funcs.module) module_delete(funcs.module);
    break;
  }
  data=NULL;
  bbtype=BLTINBUF_TYPE_UNDEF;
  return true;
};
