/* 
 * Copyright (c) 2003-2005 RIKEN Japan, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY RIKEN AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL RIKEN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: fontsize.c,v 1.2 2005/10/30 09:55:30 orrisroot Exp $ */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#ifdef WIN32
# include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libgpm.h"
#include "libgpmpriv.h"

#ifdef __cplusplus
extern "C" {
#endif

/* font size database */
static int  times_roman[] = {
  250, 333, 408, 500, 500, 833, 778, 333, /* SP ! " # $ % & ' */
  333, 333, 500, 564, 250, 333, 250, 278, /*  ( ) * + , - . / */
  500, 500, 500, 500, 500, 500, 500, 500, /*  0 1 2 3 4 5 6 7 */ 
  500, 500, 278, 278, 564, 564, 564, 444, /*  8 9 : ; < = > ? */
  921, 722, 667, 667, 722, 611, 556, 722, /*  @ A B C D E F G */
  722, 333, 389, 722, 611, 889, 722, 722, /*  H I J K L M N O */
  556, 722, 667, 556, 611, 722, 722, 944, /*  P Q R S T U V W */
  722, 722, 611, 333, 278, 333, 469, 500, /*  X Y Z [ \ ] ^ _ */
  333, 444, 500, 444, 500, 444, 333, 500, /*  ` a b c d e f g */
  500, 278, 278, 500, 278, 778, 500, 500, /*  h i j k l m n o */
  500, 500, 333, 389, 278, 500, 500, 722, /*  p q r s t u v w */
  500, 500, 444, 480, 200, 480, 541, 250};/*  x y z { | } ~   */
					     
static int  times_italic[] = {
  250, 333, 420, 500, 500, 833, 778, 333, /* SP ! " # $ % & ' */
  333, 333, 500, 675, 250, 333, 250, 278, /*  ( ) * + , - . / */
  500, 500, 500, 500, 500, 500, 500, 500, /*  0 1 2 3 4 5 6 7 */ 
  500, 500, 333, 333, 675, 675, 675, 500, /*  8 9 : ; < = > ? */
  920, 611, 611, 667, 722, 611, 611, 722, /*  @ A B C D E F G */
  722, 333, 444, 667, 556, 833, 667, 722, /*  H I J K L M N O */
  611, 722, 611, 500, 556, 722, 611, 833, /*  P Q R S T U V W */
  611, 556, 556, 389, 278, 389, 422, 500, /*  X Y Z [ \ ] ^ _ */
  333, 500, 500, 444, 500, 444, 278, 500, /*  ` a b c d e f g */
  500, 278, 278, 444, 278, 722, 500, 500, /*  h i j k l m n o */
  500, 500, 389, 389, 278, 500, 444, 667, /*  p q r s t u v w */
  444, 444, 389, 400, 275, 400, 541, 250};/*  x y z { | } ~   */

static int  times_bold[] = {
  250, 333, 555, 500, 500,1000, 833, 333, /* SP ! " # $ % & ' */
  333, 333, 500, 570, 250, 333, 250, 278, /*  ( ) * + , - . / */
  500, 500, 500, 500, 500, 500, 500, 500, /*  0 1 2 3 4 5 6 7 */ 
  500, 500, 333, 333, 570, 570, 570, 500, /*  8 9 : ; < = > ? */
  930, 722, 667, 722, 722, 667, 611, 778, /*  @ A B C D E F G */
  778, 389, 500, 778, 667, 944, 722, 778, /*  H I J K L M N O */
  611, 778, 722, 556, 667, 722, 722,1000, /*  P Q R S T U V W */
  722, 722, 667, 333, 278, 333, 581, 500, /*  X Y Z [ \ ] ^ _ */
  333, 500, 556, 444, 556, 444, 333, 500, /*  ` a b c d e f g */
  556, 278, 333, 556, 278, 833, 556, 500, /*  h i j k l m n o */
  556, 556, 444, 389, 333, 556, 500, 722, /*  p q r s t u v w */
  500, 500, 444, 394, 220, 394, 520, 250};/*  x y z { | } ~   */

static int  times_bolditalic[] = {
  250, 389, 555, 500, 500, 833, 778, 333, /* SP ! " # $ % & ' */
  333, 333, 500, 570, 250, 333, 250, 278, /*  ( ) * + , - . / */
  500, 500, 500, 500, 500, 500, 500, 500, /*  0 1 2 3 4 5 6 7 */ 
  500, 500, 333, 333, 570, 570, 570, 500, /*  8 9 : ; < = > ? */
  832, 667, 667, 667, 722, 667, 667, 722, /*  @ A B C D E F G */
  778, 389, 500, 667, 611, 889, 722, 722, /*  H I J K L M N O */
  611, 722, 667, 556, 611, 722, 667, 889, /*  P Q R S T U V W */
  667, 611, 611, 333, 278, 333, 570, 500, /*  X Y Z [ \ ] ^ _ */
  333, 500, 500, 444, 500, 444, 333, 500, /*  ` a b c d e f g */
  556, 278, 278, 500, 278, 778, 556, 500, /*  h i j k l m n o */
  500, 500, 389, 389, 278, 556, 444, 667, /*  p q r s t u v w */
  500, 444, 389, 348, 220, 348, 570, 250};/*  x y z { | } ~   */

static int  helvetica[] = {
  278, 278, 355, 556, 556, 889, 667, 222, /* SP ! " # $ % & ' */
  333, 333, 389, 584, 278, 333, 278, 278, /*  ( ) * + , - . / */
  556, 556, 556, 556, 556, 556, 556, 556, /*  0 1 2 3 4 5 6 7 */ 
  556, 556, 278, 278, 584, 584, 584, 556, /*  8 9 : ; < = > ? */
 1015, 667, 667, 722, 722, 667, 611, 778, /*  @ A B C D E F G */
  722, 278, 500, 667, 556, 833, 722, 778, /*  H I J K L M N O */
  667, 778, 722, 667, 611, 722, 667, 944, /*  P Q R S T U V W */
  667, 667, 611, 278, 278, 278, 649, 556, /*  X Y Z [ \ ] ^ _ */
  222, 556, 556, 500, 556, 556, 278, 556, /*  ` a b c d e f g */
  556, 222, 222, 500, 222, 833, 556, 556, /*  h i j k l m n o */
  556, 556, 333, 500, 278, 556, 500, 722, /*  p q r s t u v w */
  500, 500, 500, 334, 260, 334, 584, 278};/*  x y z { | } ~   */

static int helvetica_oblique[] = {
  278, 278, 355, 556, 556, 889, 667, 222, /* SP ! " # $ % & ' */
  333, 333, 389, 584, 278, 333, 278, 278, /*  ( ) * + , - . / */
  556, 556, 556, 556, 556, 556, 556, 556, /*  0 1 2 3 4 5 6 7 */ 
  556, 556, 278, 278, 584, 584, 584, 556, /*  8 9 : ; < = > ? */
 1015, 667, 667, 722, 722, 667, 611, 778, /*  @ A B C D E F G */
  722, 278, 500, 667, 556, 833, 722, 778, /*  H I J K L M N O */
  667, 778, 722, 667, 611, 722, 667, 944, /*  P Q R S T U V W */
  667, 667, 611, 278, 278, 278, 469, 556, /*  X Y Z [ \ ] ^ _ */
  222, 556, 556, 500, 556, 556, 278, 556, /*  ` a b c d e f g */
  556, 222, 222, 500, 222, 533, 556, 556, /*  h i j k l m n o */
  556, 556, 333, 500, 278, 556, 500, 722, /*  p q r s t u v w */
  500, 500, 500, 334, 260, 334, 584, 278};/*  x y z { | } ~   */

static int  helvetica_bold[] = {
  278, 333, 474, 556, 556, 889, 722, 278, /* SP ! " # $ % & ' */
  333, 333, 389, 584, 278, 333, 278, 278, /*  ( ) * + , - . / */
  556, 556, 556, 556, 556, 556, 556, 556, /*  0 1 2 3 4 5 6 7 */ 
  556, 556, 333, 333, 584, 584, 584, 611, /*  8 9 : ; < = > ? */
  975, 722, 722, 722, 722, 667, 611, 778, /*  @ A B C D E F G */
  722, 278, 556, 722, 611, 833, 722, 778, /*  H I J K L M N O */
  667, 778, 722, 667, 611, 722, 667, 944, /*  P Q R S T U V W */
  667, 667, 611, 333, 278, 333, 584, 556, /*  X Y Z [ \ ] ^ _ */
  278, 556, 611, 556, 611, 556, 333, 611, /*  ` a b c d e f g */
  611, 278, 278, 556, 278, 889, 611, 611, /*  h i j k l m n o */
  611, 611, 389, 556, 333, 611, 556, 778, /*  p q r s t u v w */
  556, 556, 500, 389, 280, 389, 584, 278};/*  x y z { | } ~   */

static int helvetica_boldoblique[] = {
  278, 333, 474, 556, 556, 889, 722, 278, /* SP ! " # $ % & ' */
  333, 333, 389, 584, 278, 333, 278, 278, /*  ( ) * + , - . / */
  556, 556, 556, 556, 556, 556, 556, 556, /*  0 1 2 3 4 5 6 7 */ 
  556, 556, 333, 333, 584, 584, 584, 611, /*  8 9 : ; < = > ? */
  975, 722, 722, 722, 722, 667, 611, 778, /*  @ A B C D E F G */
  722, 287, 556, 722, 611, 833, 722, 778, /*  H I J K L M N O */
  667, 778, 722, 667, 611, 722, 667, 944, /*  P Q R S T U V W */
  667, 667, 611, 333, 278, 333, 584, 556, /*  X Y Z [ \ ] ^ _ */
  278, 556, 611, 556, 611, 556, 333, 611, /*  ` a b c d e f g */
  611, 278, 278, 556, 278, 889, 611, 611, /*  h i j k l m n o */
  611, 611, 389, 556, 333, 611, 556, 778, /*  p q r s t u v w */
  556, 556, 500, 389, 280, 389, 584, 278};/*  x y z { | } ~   */

static int courier[] = {
  600, 600, 600, 600, 600, 600, 600, 600, /* SP ! " # $ % & ' */
  600, 600, 600, 600, 600, 600, 600, 600, /*  ( ) * + , - . / */
  600, 600, 600, 600, 600, 600, 600, 600, /*  0 1 2 3 4 5 6 7 */ 
  600, 600, 600, 600, 600, 600, 600, 600, /*  8 9 : ; < = > ? */
  600, 600, 600, 600, 600, 600, 600, 600, /*  @ A B C D E F G */
  600, 600, 600, 600, 600, 600, 600, 600, /*  H I J K L M N O */
  600, 600, 600, 600, 600, 600, 600, 600, /*  P Q R S T U V W */
  600, 600, 600, 600, 600, 600, 600, 600, /*  X Y Z [ \ ] ^ _ */
  600, 600, 600, 600, 600, 600, 600, 600, /*  ` a b c d e f g */
  600, 600, 600, 600, 600, 600, 600, 600, /*  h i j k l m n o */
  600, 600, 600, 600, 600, 600, 600, 600, /*  p q r s t u v w */
  600, 600, 600, 600, 600, 600, 600, 600};/*  x y z { | } ~   */

static int *gpmfont_width_data[] = {
  times_roman, times_italic,      times_bold,     times_bolditalic,
  helvetica,   helvetica_oblique, helvetica_bold, helvetica_boldoblique,
  courier,     courier,           courier,        courier
};


/*************************************************************/
/*  getlength()                                              */
/*************************************************************/
void getlength(int number, float *x, float *y, int rotate){
  float xtmp, ytmp;
  static float dev_length[][2] = {
    {GPM_PAPER_A4_WIDTH,     GPM_PAPER_A4_HEIGHT},   /* A4 Paper  */
    {GPM_PAPER_B4_WIDTH,     GPM_PAPER_B4_HEIGHT},   /* B4 Paper  */
    {GPM_PAPER_FREE_DEFAULT, GPM_PAPER_FREE_DEFAULT} /* Free Size */
  };
  if((number >= GPM_PAPER_A4) && (number <= GPM_PAPER_FREE)){
    if(rotate == 0){
      xtmp = dev_length[number][0];
      ytmp = dev_length[number][1];
    } else {
      xtmp = dev_length[number][1];
      ytmp = dev_length[number][0];
    }
  } else {
    xtmp = 9999.0;
    ytmp = 9999.0;
  }
  if(x) *x = xtmp;
  if(y) *y = ytmp;
}

/*************************************************************/
/*  getfontwidth()                                           */
/*************************************************************/
double getfontwidth(int c, double height, int font){
  int    *fontbuffer;
  double  length;
  fontbuffer = gpmfont_width_data[font%12];
  length = ((double)fontbuffer[c-32] * height / 1000.0);
  return length;
}


/************************************************************
 *  getstrwidth()                                            *
 ************************************************************/
double getstrwidth(const char *str, double height, int font){
  int    *fontbuffer;
  size_t  i;
  double  length;
  int     c;
  fontbuffer = gpmfont_width_data[font%12];
  length = 0;
  for(i=0; i<strlen(str); i++){
    c = *(str+i);
    length += ((double)fontbuffer[c-32] * height / 1000.0);
  }
  return length;
}

#ifdef __cplusplus
}
#endif
