/* 
 * Copyright (c) 2003 RIKEN (The Institute of Physical and Chemical Research)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY RIKEN AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL RIKEN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: det.cpp,v 1.1.1.1 2004/03/31 08:15:05 orrisroot Exp $ */
/*******************************************
 * Digital Signal Processing Command Lib.  *
 *                   Presented By K.Takebe *
 *******************************************/
/*******************************************
 *  COMMAND NAME : det                     *
 *    ( Ver 1.0 : 1993/03/19 )             *
 *******************************************/

#include <stdio.h>
#include <math.h>
#include "SL_macro.h"
#include "SL_cmd.h"

#ifdef __cplusplus
extern "C" {
#endif

#define FLOAT_MINIMUM 1.0e-15


double determ (Buffer *a, int n );

DLLEXPORT int mod_ispp_det(){
  Buffer *x, *y, *xx;
  int    index[MAX_INDEX];
  int    dim, i, l, m, n;

  x = GetSeries( 0, &dim, index );
  if ( x == NULL )
    return (4);

  switch ( dim ) {
  case 1: return (24); break;
  case 2: if ( index[1] != index[0] )
    return (7);
    ReturnScalar( determ( x, index[0] ) );
    break;
    
  default:
    if ( index[dim-2] != index[dim-1] )
      return (7);
    n = index[dim-2];
    m = index[dim-2]*index[dim-1];
    l = IndexSize(dim,index)/m;
    if ((y = AllocBuffer( l )) == NULL )
      return (8);

    for ( xx = x, i = 0; i < l ; i++, xx += m )
      y[i] = determ( xx, n );
    ReturnSeries( y , dim-2, index );
    FreeBuffer( y );
    break;
  }
  FreeBuffer( x );
  return 0;
}

double determ(Buffer *a,int n ){
  double *ai, *aj, *ak, *al, *pai, *pal;
  double piv, div, w, det = 1.0;

  for ( pai = (ai=a)+n*n-1, pal = a+n-1; ai < pai; ai += n+1, pal += n ) {
/*    printf("1;%1.1lf, %1.1lf, %1.1lf, %1.1lf\n", piv, div, w, det);*/
    for ( piv = *(aj = ak = ai); (aj += n) < pai; )
      if (((( w = *aj) > 0 ? w : -w ) > piv ))
	piv = *(ak = aj);
    if (piv == 0) 
      return (0);
/*    printf("2;%1.1lf, %1.1lf, %1.1lf, %1.1lf\n", piv, div, w, det);*/
    det *= piv*(ak != ai ? -1.0: 1.0);
/*    printf("3;%1.1lf, %1.1lf, %1.1lf, %1.1lf\n", piv, div, w, det);*/
    for ( *ak++ = *ai, *ai = piv, al = ai; al < pal;
	 w = *ak, *ak++ = *(++al), *al = w );
/*    printf("4;%1.1lf, %1.1lf, %1.1lf, %1.1lf\n", piv, div, w, det);*/
    if(piv == 0)
      continue;
    for ( aj = ai; (aj += n) < pai; )
      for ( div = *(ak = aj)/piv, al = ai; al < pal; )
	*(++ak) -= *(++al)*div;
/*    printf("5;%1.1lf, %1.1lf, %1.1lf, %1.1lf\n", piv, div, w, det);*/
  }
  return ( det*(*ai) );
}

#ifdef __cplusplus
}
#endif
